/*
 * Copyright 2023 Code Intelligence GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.code_intelligence.jazzer.mutation.api;

import com.google.errorprone.annotations.DoNotMock;

/**
 * Combines a {@link ValueMutator} with a {@link Serializer} for objects of type {@code T}.
 *
 * <p>Implementing classes SHOULD be declared final.
 *
 * <p>This is the default fully-featured mutator type. If {@code T} can be mutated fully in place,
 * consider implementing the more versatile {@link SerializingInPlaceMutator} instead.
 */
@DoNotMock("Use TestSupport#mockMutator instead")
public abstract class SerializingMutator<T> implements Serializer<T>, ValueMutator<T> {
  @Override
  public final String toString() {
    return Debuggable.getDebugString(this);
  }
}
